package org.refresh.examples.intelligence;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.examples.extlibrary.Book;
import org.eclipse.emf.examples.extlibrary.EXTLibraryFactory;
import org.eclipse.emf.examples.extlibrary.EXTLibraryPackage;
import org.eclipse.emf.examples.extlibrary.Writer;
import org.refresh.core.intelligence.emf.OCLConstraint;
import org.refresh.emf.EMFIntelligence;


/******************************************************************************
 * Copyright (c) 2007 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class EMFIntelligenceHelloWorld {

	/*
	 * The basic goal of EMFIntelligence is to
	 * derive valid endpoints for EStructural features
	 * given a set of objects (the knowledge base) and
	 * a set of constraints over relationships between
	 * objects in the knowledge base (you can associate
	 * constraints with EStructuralFeatures, arbitrary
	 * String names of relationships, etc.)
	 * 
	 * A very very simple example of using
	 * EMFIntelligence to derive valid endpoints
	 * for an EStructuralFeature.
	 */
	public void helloWorld(){
		
		//First, construct a basic EMFIntelligence knowledge base.
		//If you need to use constraints other than OCL, you can
		//install them by calling kb.getConstraintEvaluators().put(TYPE, ConstraintEvaluator)
		EMFIntelligence kb = new EMFIntelligence();
		
		//Now, create a simple constraint to add to the knowledge
		//base. This constraint will ensure that for any assignment
		//of the relationship it is bound to, the title of the
		//bound object is 'Hello World'. The variable 'target' denotes
		//the object that the relationship will be assigned to. Thus,
		//this constraint has no type associated with it and could
		//be assigned to arbitrary relationships that have a target
		//type with a 'title' property.
		OCLConstraint con1 = new OCLConstraint("target.title = 'Hello World'");
		
		//Create a factory so that we can construct books and a writer
		EXTLibraryFactory fact = EXTLibraryPackage.eINSTANCE
				.getEXTLibraryFactory();
		Book f1 = fact.createBook();
		Book f2 = fact.createBook();
		Book f3 = fact.createBook();
		
		//The writer that we are going to
		//query for valid books for
		Writer w1 = fact.createWriter();
		w1.setFirstName("The Writer of");
		w1.setLastName("Hello World");
		
		//Set the titles of our books
		f1.setTitle("Moby Dick");
		f2.setTitle("Birdsong");
		f3.setTitle("Hello World");

		//Add the books to the knowledge base
		kb.add(f1);
		kb.add(f2);
		kb.add(f3);
		
		//Add the writer to the knowledge base
		kb.add(w1);

		//This is the type that we are going to
		//be binding the constraint to. The 
		//constraint will only apply to writers.
		EClass sourceType = w1.eClass();
		
		//This is the feature (relationship) that
		//we will be binding the constraint to. Adding
		//this constraint will cause the knowledge
		//base to only return books with the title
		//Hello World when the "Books" feature of
		//a "Writer" is queried for valid endpoints.
		EStructuralFeature booksFeature = sourceType.getEStructuralFeature("books");
		
		//Post the constraint to the knowledge
		//base
		kb.add(sourceType, booksFeature, con1);
		
		//Ask EMFIntelligence to derive the valid
		//values for the feature. This should only
		//return book f3.
		List valid = kb.validTargets(w1, booksFeature);
		
		//Check to make sure things worked
		if(valid.get(0) == f3)
			System.out.println(f3.getTitle()+" is a valid book of "+w1.getFirstName()+" "+w1.getLastName());
		else
			System.out.println("Something went horribly wrong....");
	}
	
	public static void main(String[] args){
		EMFIntelligenceHelloWorld example = new EMFIntelligenceHelloWorld();
		example.helloWorld();
	}
}
