package org.refresh.examples.intelligence;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.examples.extlibrary.Book;
import org.eclipse.emf.examples.extlibrary.EXTLibraryFactory;
import org.eclipse.emf.examples.extlibrary.EXTLibraryPackage;
import org.refresh.core.intelligence.emf.OCLConstraint;
import org.refresh.emf.EMFIntelligence;
import org.refresh.groovy.GroovyConstraint;
import org.refresh.groovy.GroovyEvaluator;
import org.refresh.prolog.PrologConstraint;
import org.refresh.prolog.PrologEvaluator;


/******************************************************************************
 * Copyright (c) 2007 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class EMFIntelligenceMixedConstraintsExample {

	/*
	 * This example shows how EMFIntelligence can be used
	 * to derive valid endpoints for an EStructuralFeature
	 * that has a number of constraint types associated
	 * with it. This example mixes OCL, Prolog, and Groovy
	 * constraints on a single EStructuralFeature.
	 * 
	 * This example requires the Groovy and Prolog extensions
	 * to EMFIntelligence AND an installation of SWI Prolog.
	 * Please see the Prolog plug-in's installation instructions.
	 */
	public void helloWorld(){
		
		//First, construct a basic EMFIntelligence knowledge base.
		EMFIntelligence kb = new EMFIntelligence();
		
		
		//Add an evaluator to handle the Groovy constraints
		GroovyEvaluator eval2 = new GroovyEvaluator();
		kb.getConstraintEvaluators().put(GroovyConstraint.TYPE, eval2);
		
		//Add a Prolog evaluator for the Prolog constraints
		PrologEvaluator eval3 = new PrologEvaluator(kb);
		kb.getConstraintEvaluators().put(PrologConstraint.TYPE, eval3);
		//Prolog has to be notified as each new object is added
		//to the knowledge base so we add the PrologEvaluator as
		//a knowledge manager.
		kb.getKnowledgeManagers().add(eval3);
		
		//You could also create an evaluator for your own language
		//of choice (it's easy...the interface only requires you
		//to implement a single method)
		//FooEvaluator eval4 = new FooEvaluator();
		//kb.getConstraintEvaluators().put(FooCosntraint.TYPE, eval4);

		
		//Below, we create the same constraint in OCL, Prolog,
		//and Groovy. The constraint is an expression that must
		//hold true over the source and target of the relationship.
		//Normally, you would NOT duplicate the constraint in each
		//language. You would instead express different constraints
		//in different languages (using the language best suited
		//for each constraint). The constraints are duplicated here
		//just to show how the various languages are used.
		
		//The 'target' variable is the possible endpoint.
		OCLConstraint oclcon = new OCLConstraint("not(self.title = target.title)");
		
		//The 'self' variable is the source of the relationship (identical
		//to the self in an OCL constraint). The 'target' variable
		//will be assigned the proposed endpoint.
		GroovyConstraint groovycon = new GroovyConstraint(
				"target.title != self.title");
		
		//Prolog constraints are passed the source (the object that 
		//is the source of the relationship you are trying to
		//derive endpoints for) in the 'Self' variable. The
		//'Target' variable is the possible endpoint.
		PrologConstraint prologcon = new PrologConstraint("self_title(Self,MyTitle),self_title(Target,TTitle),not(MyTitle = TTitle)");
		
		
		//Create some objects for our EMFIntelligence knowledge base
		EXTLibraryFactory fact = EXTLibraryPackage.eINSTANCE
				.getEXTLibraryFactory();
		Book f1 = fact.createBook();
		Book f2 = fact.createBook();
		Book f3 = fact.createBook();
		f1.setTitle("foo");
		f2.setTitle("foo2");
		f3.setTitle("foo");

		//Add the objects to the knowledge base
		kb.add(f1);
		kb.add(f2);
		kb.add(f3);
		
		
		//We are going to place a constraint on the Book class
		EClass sourceClass = f1.eClass();
		
		//We are going to use a made up relationship between
		//Books. You can assign constraints to EStructuralFeatures -
		//but in this case we are going to invent a relationship
		//between books called "differentBook" and associated the
		//constraint with it.
		String differentBook = "differentBook";
		
		//Since we are inventing a relationship, we have to
		//define a type for the endpoints of this fictious
		//relationship. If you use a real EStructuralFeature,
		//EMFIntelligence does this automatically for you
		//by examining the EStructuralFeature.
		//In this case, our relationship is between two
		//books.
		EClass targetClass = f1.eClass();
		
		//Add the constraints to the knowledge base
		//note, we have to pass in the targetClass since
		//we aren't using an EStructuralFeature
		kb.add(sourceClass, differentBook, targetClass, oclcon);
		kb.add(sourceClass, differentBook, targetClass, prologcon);
		kb.add(sourceClass, differentBook, targetClass, groovycon);

		//Now, ask EMFIntelligence to derive valid endpoints for
		//the differentBook relationship
		List valid = kb.validTargets(f1, differentBook);
		
		if(valid.size() == 1 && valid.get(0) == f2)
			System.out.println("It worked, f2 does not have the same title as f1");
		else
			System.out.println("Something went horribly wrong...");
	}
	
	public static void main(String[] args){
		EMFIntelligenceMixedConstraintsExample example = new EMFIntelligenceMixedConstraintsExample();
		example.helloWorld();
	}
}
