package org.refresh.examples.intelligence;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.examples.extlibrary.Book;
import org.eclipse.emf.examples.extlibrary.BookCategory;
import org.eclipse.emf.examples.extlibrary.BookOnTape;
import org.eclipse.emf.examples.extlibrary.EXTLibraryFactory;
import org.eclipse.emf.examples.extlibrary.EXTLibraryPackage;
import org.eclipse.emf.examples.extlibrary.Writer;
import org.refresh.core.intelligence.emf.OCLConstraint;
import org.refresh.emf.EMFIntelligence;
import org.refresh.emf.EMFIntelligenceFactory;
import org.refresh.groovy.GroovyConstraint;
import org.refresh.groovy.GroovyEvaluator;
import org.refresh.prolog.PrologConstraint;
import org.refresh.prolog.PrologEvaluator;


/******************************************************************************
 * Copyright (c) 2007 Jules White.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Jules White - initial API and implementation 
 ****************************************************************************/
public class EMFIntelligenceWriterMixedConstraintsExample  {

	/*
	 * The basic goal of EMFIntelligence is to
	 * derive valid endpoints for EStructural features
	 * given a set of objects (the knowledge base) and
	 * a set of constraints over relationships between
	 * objects in the knowledge base (you can associate
	 * constraints with EStructuralFeatures, arbitrary
	 * String names of relationships, etc.)
	 * 
	 * A very very simple example of using
	 * EMFIntelligence to derive valid endpoints
	 * for an EStructuralFeature.
	 */
	public void helloWorld(){
		
		//First, construct a basic EMFIntelligence knowledge base.
		//If you need to use constraints other than OCL, you can
		//install them by calling kb.getConstraintEvaluators().put(TYPE, ConstraintEvaluator)
		//This factory method will automatically load all ConstraintEvaluators
		//and ObjectTypeSystems that are provided as plugins to GEMS EMF Intelligence.
		EMFIntelligence kb = EMFIntelligenceFactory.createKB();
		
		//If you this was run as an Eclipse plugin, the above line would
		//take care of this. However, since we can't guarantee that the
		//GroovyEvaluator or PrologEvaluator is installed, we just add them here to be safe.
		kb.getConstraintEvaluators().put(GroovyConstraint.TYPE, new GroovyEvaluator());
		
		//Prolog has to have a reference to the KB
		PrologEvaluator peval = new PrologEvaluator(kb);
		//Prolog also gets added as a KB manager since it
		//has to incrementally synchronize its Prolog KB
		//with Refreh's KB
		kb.getKnowledgeManagers().add(peval);
		kb.getConstraintEvaluators().put(PrologConstraint.TYPE,peval);
		
		//Now, create a simple constraint to add to the knowledge
		//base. This constraint will ensure that for any assignment
		//of the relationship it is bound to, the title of the
		//bound object is 'Hello World'. The variable 'target' denotes
		//the object that the relationship will be assigned to. Thus,
		//this constraint has no type associated with it and could
		//be assigned to arbitrary relationships that have a target
		//type with a 'title' property.
		OCLConstraint con1 = new OCLConstraint("self.books->exists( b : Book | b.category->includes(target.category))");
		GroovyConstraint gcon = new GroovyConstraint("target.title.contains(\"\"+self.name.hashCode())");
		PrologConstraint pcon = new PrologConstraint("self_title(Target,N),self_title(Tape,N),is_a(Tape,bookontape)");
		
		//Create a factory so that we can construct books and a writer
		EXTLibraryFactory fact = EXTLibraryPackage.eINSTANCE
				.getEXTLibraryFactory();
		Book f1 = fact.createBook();
		f1.setCategory(BookCategory.MYSTERY_LITERAL);
		Book f2 = fact.createBook();
		f2.setCategory(BookCategory.BIOGRAPHY_LITERAL);
		Book f3 = fact.createBook();
		f3.setCategory(BookCategory.MYSTERY_LITERAL);
		BookOnTape tape = fact.createBookOnTape();
		
		//The writer that we are going to
		//query for valid books for
		Writer w1 = fact.createWriter();
		w1.setFirstName("The Writer of");
		w1.setLastName("Mystery");
		w1.getBooks().add(f1);
		
		//Set the titles of our books
		//We add the writer's name's hashcode
		//to the first book.
		f1.setTitle("Mystery A "+w1.getName().hashCode());
		f2.setTitle("Biography B");
		f3.setTitle("Mystery C");
		tape.setTitle("Mystery A "+w1.getName().hashCode());
		

		//Add the books to the knowledge base
		kb.add(f1);
		kb.add(f2);
		kb.add(f3);
		kb.add(tape);
		
		//Add the writer to the knowledge base
		kb.add(w1);
		
		//Sometimes, it can help to see the Prolog
		//representation of the knowledge base. This
		//call just serializes the Prolog knowledge
		//base into a series of Prolog statements
		//and prints it to standard out.
		try{
			System.out.println("############The Prolog KB############");
			peval.serializeKB(System.out);
		}catch (Exception e) {
			//We shouldn't end up here...
		}

		//This is the type that we are going to
		//be binding the constraint to. The 
		//constraint will only apply to writers.
		EClass sourceType = w1.eClass();
		
		//This is the feature (relationship) that
		//we will be binding the constraint to. Adding
		//this constraint will cause the knowledge
		//base to only return books with the title
		//Hello World when the "Books" feature of
		//a "Writer" is queried for valid endpoints.
		EStructuralFeature booksFeature = sourceType.getEStructuralFeature("books");
		
		//Post the constraint to the knowledge
		//base
		kb.add(sourceType, booksFeature, con1);
		kb.add(sourceType, booksFeature, gcon);
		kb.add(sourceType, booksFeature, pcon);
		
		//Ask EMFIntelligence to derive the valid
		//values for the feature. This should only
		//return book f1.
		List valid = kb.validTargets(w1, booksFeature);
		
		//Check to make sure things worked
		if(valid.size() == 1)
			System.out.println("It worked, the writer wrote "+((Book)valid.get(0)).getTitle());
		else
			System.out.println("Something went horribly wrong....");
	}
	
	public static void main(String[] args){
		EMFIntelligenceWriterMixedConstraintsExample example = new EMFIntelligenceWriterMixedConstraintsExample();
		example.helloWorld();
	}
}
