package org.refresh.tests;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.refresh.core.RefreshEngine;
import org.refresh.core.conf.ProblemParser;
import org.refresh.core.conf.RefreshProblem;
import org.refresh.core.conf.EvaluateHandler.AddEvaluationAction;
import org.refresh.core.expr.BinaryExpression;
import org.refresh.core.expr.Cardinality;
import org.refresh.core.expr.ConstantIntegerExpression;
import org.refresh.core.expr.SourceVariableSumExpression;
import org.refresh.core.expr.TargetVariableSumExpression;
import org.refresh.core.expr.TargetVariableValueExpression;
import org.refresh.core.intelligence.KBConstraintFactory;
import org.refresh.core.intelligence.KBFactory;
import org.refresh.groovy.GroovyConstraint;
import org.refresh.groovy.GroovyEvaluator;

/*******************************************************************************
 * Copyright (c) 2007 Jules White. All rights reserved. This program and
 * the accompanying materials are made available under the terms of the Eclipse
 * Public License v1.0 which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: Jules White - initial API and implementation
 ******************************************************************************/
public class ProblemParserTest extends TestCase {

	public void testRequiresHandler(){
		String conf = "System { Requires:A,B,C; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertTrue(problem.getRequiresMap().get("System") != null);
		assertTrue(problem.getRequiresMap().get("System") instanceof List);
		assertTrue(((List)problem.getRequiresMap().get("System")).size() == 3);
	}
	
	public void testExcludesHandler(){
		String conf = "System { Excludes:A,B,C; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertTrue(problem.getExcludesMap().get("System") != null);
		assertTrue(problem.getExcludesMap().get("System") instanceof List);
		assertTrue(((List)problem.getExcludesMap().get("System")).size() == 3);
	}
	
	public void testSelectHandler(){
		String conf = "System { Select:[1..3],A,B,C; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertTrue(problem.getSelectMap().get("System") != null);
		assertTrue(problem.getSelectMap().get("System") instanceof List);
		assertTrue(((List)problem.getSelectMap().get("System")).get(0) instanceof Cardinality);
		assertTrue(((List)problem.getSelectMap().get("System")).size() == 4);
	}
	
	public void testEvaluateHandler(){
		String conf = "System { Evaluate:groovy{target.arch == \"x86\"}; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		problem.setTargetItems(new String[]{"A"});
		problem.getTargetVariables("A").put("arch", "x86");
		assertTrue(problem.getPreConfigurationActions().size() == 1);
		
		KBConstraintFactory.getInstance().register(new GroovyConstraint());
		KBFactory.getInstance().getConstraintEvaluators().put("groovy", new GroovyEvaluator());
		RefreshEngine engine = new RefreshEngine(problem);
		problem.inject(engine);
		
		AddEvaluationAction act = (AddEvaluationAction)problem.getPreConfigurationActions().get(0);
		assertEquals("target.arch == \"x86\"", act.getExpression());
		assertEquals("groovy", act.getExpressionType());
	
		assertEquals(1,problem.getFeasibleTargets("System").size());
		assertEquals("A",problem.getFeasibleTargets("System").get(0));
		
		conf = "System { Evaluate:groovy{target.arch == \"ARM\"}; }";
		problem = parser.parseProblem(conf);
		problem.setTargetItems(new String[]{"A"});
		problem.getTargetVariables("A").put("arch", "x86");
		assertTrue(problem.getPreConfigurationActions().size() == 1);
		
		KBConstraintFactory.getInstance().register(new GroovyConstraint());
		KBFactory.getInstance().getConstraintEvaluators().put("groovy", new GroovyEvaluator());
		engine = new RefreshEngine(problem);
		problem.inject(engine);
		
		act = (AddEvaluationAction)problem.getPreConfigurationActions().get(0);
		assertEquals("target.arch == \"ARM\"", act.getExpression());
		assertEquals("groovy", act.getExpressionType());
	
		assertEquals(0,problem.getFeasibleTargets("System").size());
		
	}
	
	public void testRequiredHandler(){
		String conf = "System { Required:true; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertTrue(problem.getSourceMappedInstancesCountMap().get("System") != null);
		assertTrue(problem.getSourceMappedInstancesCountMap().get("System").getMin() == 1);
		assertTrue(problem.getSourceMappedInstancesCountMap().get("System").getMax() == 1);
	}
	
	public void testTargetReqHandler(){
		String conf = "System { Target:Target.isrunning = 1; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertTrue(problem.getSourceFeasibilityConstraintsMap().get("System")!=null);
		assertTrue(problem.getSourceFeasibilityConstraintsMap().get("System").size() == 1);
		assertTrue(problem.getSourceFeasibilityConstraintsMap().get("System").get(0).getExpression() instanceof BinaryExpression);
		BinaryExpression expr = (BinaryExpression)problem.getSourceFeasibilityConstraintsMap().get("System").get(0).getExpression();
		assertTrue(expr.getLeftHandSide() instanceof TargetVariableValueExpression);
		assertTrue(expr.getRightHandSide() instanceof ConstantIntegerExpression);
		assertTrue(expr.getOperation().equals("="));
	}
	
	public void testGoalHandler(){
		String conf = "System { Goal:Target.RAM.Sum - Source.RAM.Sum; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);

		BinaryExpression expr = (BinaryExpression)problem.getGoalFunction();
		assertNotNull(expr);
		assertTrue(expr.getLeftHandSide() instanceof TargetVariableSumExpression);
		assertTrue(expr.getRightHandSide() instanceof SourceVariableSumExpression);
		assertTrue(expr.getOperation().equals("-"));
		assertTrue(problem.isMaximizeGoal());
	}
	
	public void testDefineHandler(){
		String conf = "System { Define:A=43; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertNotNull(problem.getSourceVariableValuesTable().get("System"));
		assertNotNull(problem.getSourceVariableValuesTable().get("System").get("A"));
		assertEquals(problem.getSourceVariableValuesTable().get("System").get("A"),"43");
	}
	
	public void testSetHandler(){
		String conf = "System { Set:System.platform=Linux; Set:System.ram=300;}";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertNotNull(problem.getInjectionValues("System"));
		assertNotNull(problem.getInjectionValues("System").get("System.platform"));
		assertNotNull(problem.getInjectionValues("System").get("System.ram"));
		assertEquals("Linux",problem.getInjectionValues("System").get("System.platform"));
		assertEquals("300",problem.getInjectionValues("System").get("System.ram"));
	}
	
	public void testSetHandlerWithList(){
		String conf = "System { Set:System.platform=[Linux,Windows]; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertNotNull(problem.getInjectionValues("System"));
		assertNotNull(problem.getInjectionValues("System").get("System.platform"));
		assertEquals(ArrayList.class,problem.getInjectionValues("System").get("System.platform").getClass());
		assertEquals(2,((List)problem.getInjectionValues("System").get("System.platform")).size());
		assertEquals("Linux",((List)problem.getInjectionValues("System").get("System.platform")).get(0));
		assertEquals("Windows",((List)problem.getInjectionValues("System").get("System.platform")).get(1));
		
	}
	
	public void testSetHandlerWithMapList(){
		String conf = "System { Set:System.platform=[{a=Linux;b=Windows}]; }";
		ProblemParser parser = new ProblemParser();
		RefreshProblem problem = parser.parseProblem(conf);
		assertNotNull(problem.getInjectionValues("System"));
		assertNotNull(problem.getInjectionValues("System").get("System.platform"));
		assertEquals(ArrayList.class,problem.getInjectionValues("System").get("System.platform").getClass());
		assertEquals(1,((List)problem.getInjectionValues("System").get("System.platform")).size());
		
		Object omap = ((List)problem.getInjectionValues("System").get("System.platform")).get(0);
		assertEquals(HashMap.class, omap.getClass());
		Map map = (Map)omap;
		assertEquals("Linux",map.get("a"));
		assertEquals("Windows",map.get("b"));
		
	}
	

}
